/*
 * Copyright (c) 1995, 1996, 1997, 1998 by Internet Software Consortium
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET SOFTWARE
 * CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/* eventlib.h - exported interfaces for eventlib
 * vix 09sep95 [initial]
 *
 * $Id: eventlib.h,v 1.5.186.1.4.1 2009-08-13 06:09:35 kdickman Exp $
 */

#ifndef __ISC_EVENTLIB_H__
#define __ISC_EVENTLIB_H__

#include <sys/types.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <stdio.h>

#define	EVENTLIB_SYNC_HANDLER	1

/* In the absence of branded types... */
typedef struct { void *opaque; } evConnID;
typedef struct { void *opaque; } evFileID;
typedef struct { void *opaque; } evStreamID;
typedef struct { void *opaque; } evTimerID;
typedef struct { void *opaque; } evWaitID;
typedef struct { void *opaque; } evContext;
typedef struct { void *opaque; } evEvent;

#define evInitID(id) ((id)->opaque = NULL)
#define evTestID(id) ((id).opaque != NULL)

typedef void (*evConnFunc)__P((evContext ctx, void *uap, int fd,
			       const void *la, int lalen,
			       const void *ra, int ralen));
typedef void (*evFileFunc)__P((evContext ctx, void *uap, int fd, int evmask));
typedef	void (*evStreamFunc)__P((evContext ctx, void *uap, int fd, int bytes));
typedef void (*evTimerFunc)__P((evContext ctx, void *uap,
				struct timespec due, struct timespec inter));
typedef	void (*evWaitFunc)__P((evContext ctx, void *uap, const void *tag));
#ifdef EVENTLIB_SYNC_HANDLER
typedef int (*evSyncFunc) __P((evContext ctx, void *uap));
#endif /* EVENTLIB_SYNC_HANDLER */

typedef	struct { unsigned char mask[256/8]; } evByteMask;
#define	EV_BYTEMASK_BYTE(b) ((b) / 8)
#define	EV_BYTEMASK_MASK(b) (1 << ((b) % 8))
#define	EV_BYTEMASK_SET(bm, b) \
	((bm).mask[EV_BYTEMASK_BYTE(b)] |= EV_BYTEMASK_MASK(b))
#define	EV_BYTEMASK_CLR(bm, b) \
	((bm).mask[EV_BYTEMASK_BYTE(b)] &= ~EV_BYTEMASK_MASK(b))
#define	EV_BYTEMASK_TST(bm, b) \
	((bm).mask[EV_BYTEMASK_BYTE(b)] & EV_BYTEMASK_MASK(b))

#define	EV_POLL		1
#define	EV_WAIT		2
#define	EV_NULL		4

#define	EV_READ		1
#define	EV_WRITE	2
#define	EV_EXCEPT	4

__BEGIN_DECLS
/* eventlib.c */
#define evCreate	__evCreate
#define evSetDebug	__evSetDebug
#define evDestroy	__evDestroy
#define evGetNext	__evGetNext
#define evDispatch	__evDispatch
#define evDrop		__evDrop
#define evMainLoop	__evMainLoop
#define evGetFDs	__evGetFDs
#define evGetEventInfo	__evGetEventInfo

#ifdef EVENTLIB_SYNC_HANDLER
#define evSetSyncHandler __evSetSyncHandler
#define evResetSyncHandler __evResetSyncHandler
#define evMainLoopSync   __evMainLoopSync
#define evMainLoopSyncSighdl   __evMainLoopSyncSighdl
#endif /* EVENTLIB_SYNC_HANDLER */

extern int  evCreate __P((evContext *ctx));
extern void evSetDebug __P((evContext ctx, int lev, FILE *out));
extern int  evDestroy __P((evContext ctx));
extern int  evGetNext __P((evContext ctx, evEvent *ev, int options));
extern int  evDispatch __P((evContext ctx, evEvent ev));
extern void evDrop __P((evContext ctx, evEvent ev));
extern int  evMainLoop __P((evContext ctx));
extern int  evGetFDs __P((evContext ctx, fd_set *rd, fd_set *wr, fd_set *ex, int *max_fd, int set_size));
extern void evGetEventInfo __P((evEvent ev, void **func, int *type));

#ifdef EVENTLIB_SYNC_HANDLER
extern int  evSetSyncHandler __P((evContext ctx, evSyncFunc func, void * uap));
extern int  evResetSyncHandler __P((evContext ctx));
extern int  evMainLoopSync __P((evContext ctx));
extern int  evMainLoopSyncSighdl __P((evContext ctx));
#endif /* EVENTLIB_SYNC_HANDLER */

/* ev_connects.c */
#define evListen	__evListen
#define evConnect	__evConnect
#define evCancelConn	__evCancelConn
#define evHold		__evHold
#define evUnhold	__evUnhold
#define evTryAccept	__evTryAccept

extern int evListen __P((evContext ctx, int fd, int maxconn,
		  	 evConnFunc func, void *uap, evConnID *id));
extern int evConnect __P((evContext ctx, int fd, void *ra, int ralen,
		   	  evConnFunc func, void *uap, evConnID *id));
extern int evCancelConn __P((evContext ctx, evConnID id));
extern int evHold __P((evContext, evConnID));
extern int evUnhold __P((evContext, evConnID));
extern int evTryAccept __P((evContext, evConnID, int *));

/* ev_files.c */
#define evSelectFD	__evSelectFD
#define evDeselectFD	__evDeselectFD

extern int evSelectFD __P((evContext ctx, int fd, int eventmask,
		    	   evFileFunc func, void *uap, evFileID *id));
extern int evDeselectFD __P((evContext ctx, evFileID id));

/* ev_streams.c */
#define evConsIovec	__evConsIovec
#define evWrite		__evWrite
#define evRead		__evRead
#define evTimeRW	__evTimeRW
#define evUntimeRW	__evUntimeRW
#define	evCancelRW	__evCancelRW

extern struct iovec evConsIovec __P((void *buf, size_t cnt));
extern int evWrite __P((evContext ctx, int fd, const struct iovec *iov, int cnt,
		 	evStreamFunc func, void *uap, evStreamID *id));
extern int evRead __P((evContext ctx, int fd, const struct iovec *iov, int cnt,
		       evStreamFunc func, void *uap, evStreamID *id));
extern int evTimeRW __P((evContext ctx, evStreamID id, evTimerID timer));
extern int evUntimeRW __P((evContext ctx, evStreamID id));
extern int evCancelRW __P((evContext ctx, evStreamID id));

/* ev_timers.c */
#define evConsTime	__evConsTime
#define evAddTime	__evAddTime
#define evSubTime	__evSubTime
#define evCmpTime	__evCmpTime
#define	evTimeSpec	__evTimeSpec
#define	evTimeVal	__evTimeVal

#define evNowTime		__evNowTime
#define evLastEventTime		__evLastEventTime
#define evSetTimer		__evSetTimer
#define evClearTimer		__evClearTimer
#define evResetTimer		__evResetTimer
#define evSetIdleTimer		__evSetIdleTimer
#define evClearIdleTimer	__evClearIdleTimer
#define evResetIdleTimer	__evResetIdleTimer
#define evTouchIdleTimer	__evTouchIdleTimer

extern struct timespec evConsTime __P((time_t sec, long nsec));
extern struct timespec evAddTime __P((struct timespec add1,
				      struct timespec add2));
extern struct timespec evSubTime __P((struct timespec minu,
				      struct timespec subtra));
extern struct timespec evNowTime __P((void));
extern struct timespec evLastEventTime __P((evContext));
extern struct timespec evTimeSpec __P((struct timeval));
extern struct timeval evTimeVal __P((struct timespec));
extern struct timespec evTimeLeft __P((evTimerID opaqueID));
extern int evCmpTime __P((struct timespec a, struct timespec b));
extern int evSetTimer __P((evContext ctx, evTimerFunc func, void *uap,
			   struct timespec due, struct timespec inter,
		    	   evTimerID *id));
extern int evClearTimer __P((evContext ctx, evTimerID id));
extern int evResetTimer __P((evContext, evTimerID, evTimerFunc, void *,
		      	     struct timespec, struct timespec));
extern int evSetIdleTimer __P((evContext, evTimerFunc, void *, struct timespec,
			       evTimerID *));
extern int evClearIdleTimer __P((evContext, evTimerID));
extern int evResetIdleTimer __P((evContext, evTimerID, evTimerFunc, void *,
			  	 struct timespec));
extern int evTouchIdleTimer __P((evContext, evTimerID));

/* ev_waits.c */
#define evWaitFor	__evWaitFor
#define evDo		__evDo
#define evUnwait	__evUnwait
#define evDefer		__evDefer

extern int evWaitFor __P((evContext ctx, const void *tag, evWaitFunc func,
			  void *uap, evWaitID *id));
extern int evDo __P((evContext ctx, const void *tag));
extern int evUnwait __P((evContext ctx, evWaitID id));
extern int evDefer __P((evContext, evWaitFunc, void *));
__END_DECLS

#endif /* __ISC_EVENTLIB_H__ */
