/*
 * Copyright (c) 1997 by Internet Software Consortium.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET SOFTWARE
 * CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * $Id: heap.h,v 1.2 1998-05-13 00:47:27 pst Exp $
 */

#ifndef __ISC_HEAP_H__
#define __ISC_HEAP_H__

typedef int (*heap_higher_priority_func) __P((void *, void *));
typedef void (*heap_index_func) __P((void *, int));
typedef void (*heap_for_each_func) __P((void *, void *));

typedef struct heap_context {
	int array_size;
	int array_size_increment;
	int heap_size;
	void **heap;
	heap_higher_priority_func higher_priority;
	heap_index_func index;
} *heap_context;

#define heap_new	__heap_new
#define heap_free	__heap_free
#define heap_insert	__heap_insert
#define heap_delete	__heap_delete
#define heap_increased	__heap_increased
#define heap_decreased	__heap_decreased
#define heap_element	__heap_element
#define heap_for_each	__heap_for_each

__BEGIN_DECLS
extern heap_context	heap_new __P((heap_higher_priority_func,
				      heap_index_func, int));
extern int		heap_free __P((heap_context));
extern int		heap_insert __P((heap_context, void *));
extern int		heap_delete __P((heap_context, int));
extern int		heap_increased __P((heap_context, int));
extern int		heap_decreased __P((heap_context, int));
extern void *		heap_element __P((heap_context, int));
extern int		heap_for_each __P((heap_context, heap_for_each_func,
					   void *));
__END_DECLS

#endif /* __ISC_HEAP_H__ */
