/*
 * Copyright (c) 1996, 1997, 1998 by Internet Software Consortium.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET SOFTWARE
 * CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * $Id: logging.h,v 1.3 2007-04-01 03:15:02 subhashg Exp $
 */

#ifndef __ISC_LOGGING_H__
#define __ISC_LOGGING_H__

#include <stdio.h>
#include <stdarg.h>

#define log_critical		(-5)
#define log_error   		(-4)
#define log_warning 		(-3)
#define log_notice  		(-2)
#define log_info  		(-1)
#define log_debug(level)	(level)

typedef enum { log_syslog, log_file, log_null } log_channel_type;

#define LOG_MAX_VERSIONS 99

#define LOG_CLOSE_STREAM		0x0001
#define LOG_TIMESTAMP			0x0002
#define LOG_TRUNCATE			0x0004
#define LOG_USE_CONTEXT_LEVEL		0x0008
#define LOG_PRINT_LEVEL			0x0010
#define LOG_REQUIRE_DEBUG		0x0020
#define LOG_CHANNEL_BROKEN		0x0040
#define LOG_PRINT_CATEGORY		0x0080
#define LOG_CHANNEL_OFF			0x0100

typedef struct log_context *log_context;
typedef struct log_channel *log_channel;

#define LOG_OPTION_DEBUG		0x01
#define LOG_OPTION_LEVEL		0x02

#define log_open_stream		__log_open_stream
#define log_close_stream	__log_close_stream
#define log_get_stream		__log_get_stream
#define log_get_filename	__log_get_filename
#define log_check_channel	__log_check_channel
#define log_check		__log_check
#define log_vwrite		__log_vwrite
#define log_write		__log_write
#define log_new_context		__log_new_context
#define log_free_context	__log_free_context
#define log_add_channel		__log_add_channel
#define log_remove_channel	__log_remove_channel
#define log_option		__log_option
#define log_category_is_active	__log_category_is_active
#define log_new_syslog_channel	__log_new_syslog_channel
#define log_new_file_channel	__log_new_file_channel
#define log_new_null_channel	__log_new_null_channel
#define log_inc_references	__log_inc_references
#define log_dec_references	__log_dec_references
#define log_get_channel_type	__log_get_channel_type
#define log_free_channel	__log_free_channel

__BEGIN_DECLS
extern FILE *		log_open_stream __P((log_channel));
extern int		log_close_stream __P((log_channel));
extern FILE *		log_get_stream __P((log_channel));
extern char *		log_get_filename __P((log_channel));
extern int		log_check_channel __P((log_context, int, log_channel));
extern int		log_check __P((log_context, int, int));
extern void		log_vwrite __P((log_context, int, int, const char *, 
					va_list args));
extern void		log_write __P((log_context, int, int, const char *, ...));
extern int		log_new_context __P((int, char **, log_context *));
extern void		log_free_context __P((log_context));
extern int		log_add_channel __P((log_context, int, log_channel));
extern int		log_remove_channel __P((log_context, int, log_channel));
extern int		log_option __P((log_context, int, int));
extern int		log_category_is_active __P((log_context, int));
extern log_channel	log_new_syslog_channel __P((unsigned int, int, int));
extern log_channel	log_new_file_channel __P((unsigned int, int, char *,
	 					  FILE *, unsigned int,
						  unsigned long));
extern log_channel	log_new_null_channel __P((void));
extern int		log_inc_references __P((log_channel));
extern int		log_dec_references __P((log_channel));
extern log_channel_type	log_get_channel_type __P((log_channel));
extern int		log_free_channel __P((log_channel));
__END_DECLS

#endif /* __ISC_LOGGING_H__ */
